/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung - Bibliothek
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.lib;

import java.util.ArrayList;
import java.util.Collection;

/**
 * Exception, die ausgelöst wird, wenn ein Zugriff auf die Parametrierung nicht
 * korrekt ausgeführt werden konnte.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ParameterClientException extends Exception {

	/** Versions-ID für die Serialisierung. */
	private static final long serialVersionUID = 1L;

	/** die Liste der fehlerhaften ParameterIdentifikationen. */
	private final Collection<ParameterInfo> fehlerListe = new ArrayList<>();

	/**
	 * erzeugt eine Exception mit dem angegeben Meldungstext.
	 *
	 * @param meldung
	 *            der Meldungstext
	 */
	ParameterClientException(final String meldung) {
		this(meldung, (ParameterInfo) null);
	}

	/**
	 * erzeugt eine Exception mit dem angegeben Meldungstext und der Liste von
	 * fehlerhaften Parameteridentifikationen.
	 *
	 * @param meldung
	 *            der Meldungstext
	 * @param fehlerListe
	 *            die fehlerhaften Parameteridentifikationen
	 */
	ParameterClientException(final String meldung, final Collection<ParameterInfo> fehlerListe) {
		this(meldung, fehlerListe.toArray(new ParameterInfo[fehlerListe.size()]));
	}

	/**
	 * erzeugt eine Exception mit dem angegeben Meldungstext und der Menge von
	 * fehlerhaften Parameteridentifikationen.
	 *
	 * @param meldung
	 *            der Meldungstext
	 * @param fehlerListe
	 *            die fehlerhaften Parameteridentifikationen
	 */
	private ParameterClientException(final String meldung, final ParameterInfo... fehlerListe) {
		super(meldung);
		if (null != fehlerListe) {
			for (final ParameterInfo info : fehlerListe) {
				this.fehlerListe.add(info);
			}
		}
	}

	/**
	 * erzeugt eine Exception mit dem angegeben Meldungstext und der übergebenen
	 * Ursache.
	 *
	 * @param meldung
	 *            die Meldung
	 * @param ursache
	 *            die Ursache
	 */
	ParameterClientException(final String meldung, final Throwable ursache) {
		super(meldung, ursache);
	}

	/**
	 * liefert die Liste der fehlerhaften Parameteridentidentifikationen.
	 *
	 * @return fehlerListe
	 */
	public Collection<ParameterInfo> getFehlerListe() {
		return fehlerListe;
	}
}
